function [output_image] = Prefilter(input_image, viewpos, noise_param, noise_type)

[h,w] = size(input_image);
output_image = zeros(h,w);
prefilter = filtershape(w, viewpos, noise_param, noise_type);

for idy = 1:h
    FFT_input= fftshift(fft(input_image(idy,:)));
    FFT_processed = FFT_input .* prefilter;
    output_image(idy,:) = real(ifft(fftshift((FFT_processed))));
end
    
function output_shape = filtershape(filter_length, viewpos, noise_param, noise_type)

output_shape = zeros(1,filter_length);
for idx = 1:filter_length
    omega = (idx - floor(filter_length/2) - 1)/filter_length * (2*pi);
    FDw = func_psi(noise_param*omega, noise_type);
    FDsw = func_psi(noise_param*omega*viewpos, noise_type);
    FD1sw = func_psi(noise_param*omega*(1-viewpos), noise_type);
    response = (FDsw - FDw*FD1sw)/(1 - FDw*FDw);
    if(isnan(response))
        response = 1 - viewpos;
    end
    output_shape(idx) = response;
end

function y = func_psi(x, noise_type)
if strcmp(noise_type,'gaussian')
    y = exp( - x.^2/2);
elseif strcmp(noise_type,'uniform')
    y = sin(x + eps.*(x == 0))./(x + eps.*(x == 0));
else
    y = 1;
end